@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms, as
  * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -28,13 +29,11 @@
 #include "base/file_utilities.h"
 
 #include <errno.h>
-#ifdef _MSC_VER
-#include <windows.h>
-#else
-#include <unistd.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/wait.h>
+#ifndef _MSC_VER
+  #include <unistd.h>
+  #include <signal.h>
+  #include <sys/types.h>
+  #include <sys/wait.h>
 #endif
 
 #include <sstream>
@@ -48,67 +47,14 @@ DEFAULT_LOG_DOMAIN("SSH tunnel")
 using namespace wb;
 using namespace base;
 
-class tunnel_auth_error : public std::runtime_error {
-public:
-  tunnel_auth_error(const std::string &err)
-      : std::runtime_error(err) {
-  }
-};
-
-class tunnel_auth_retry : public std::runtime_error {
-public:
-  tunnel_auth_retry(const std::string &err)
-      : std::runtime_error(err) {
-  }
-};
-
-class tunnel_auth_cancelled : public std::runtime_error {
-public:
-  tunnel_auth_cancelled(const std::string &err)
-      : std::runtime_error(err) {
-  }
-};
-
-class tunnel_auth_key_error : public std::runtime_error {
-public:
-  tunnel_auth_key_error(const std::string &err)
-      : std::runtime_error(err) {
-  }
-};
-
-class SSHTunnel : public sql::TunnelConnection {
-  TunnelManager *_tm;
-  int _port;
-  ssh::SSHConnectionConfig _config;
-
-public:
-  SSHTunnel(TunnelManager *tm, int port, const ssh::SSHConnectionConfig &config)
-      : _tm(tm), _port(port), _config(config) {
-    _tm->portUsageIncrement(_config);
-  }
-
-  virtual ~SSHTunnel() {
-    disconnect();
-  }
-
-  virtual int get_port() {
-    return _port;
-  }
-
-  virtual void connect(db_mgmt_ConnectionRef connectionProperties) {
-    if (_port == 0)
-      throw std::runtime_error("Could not connect SSH tunnel");
-  }
-
-  virtual void disconnect() {
-    _tm->portUsageDecrement(_config);
-  }
-};
+//----------------------------------------------------------------------------------------------------------------------
 
 TunnelManager::TunnelManager()
     : _manager(nullptr) {
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void TunnelManager::start() {
   if (_manager == nullptr)
     _manager = new ssh::SSHTunnelManager();
@@ -119,6 +65,8 @@ void TunnelManager::start() {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void TunnelManager::shutdown() {
   if (_manager != nullptr) {
     _manager->setStop();
@@ -126,6 +74,8 @@ void TunnelManager::shutdown() {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void TunnelManager::portUsageIncrement(const ssh::SSHConnectionConfig &config) {
   logDebug2("Increment port usage count: %d\n", config.localport);
   base::MutexLock lock(_usageMapMtx);
@@ -137,6 +87,8 @@ void TunnelManager::portUsageIncrement(const ssh::SSHConnectionConfig &config) {
   }
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 void TunnelManager::portUsageDecrement(const ssh::SSHConnectionConfig &config) {
   logDebug2("Decrement port usage count: %d\n", config.localport);
   base::MutexLock lock(_usageMapMtx);
@@ -150,7 +102,9 @@ void TunnelManager::portUsageDecrement(const ssh::SSHConnectionConfig &config) {
   }
 }
 
-std::shared_ptr<sql::TunnelConnection> TunnelManager::createTunnel(db_mgmt_ConnectionRef connectionProperties) {
+//----------------------------------------------------------------------------------------------------------------------
+
+std::shared_ptr<SSHTunnel> TunnelManager::createTunnel(db_mgmt_ConnectionRef connectionProperties) {
   grt::DictRef parameter_values = connectionProperties->parameterValues();
 
   if (connectionProperties->driver()->name() == "MysqlNativeSSH") {
@@ -168,7 +122,7 @@ std::shared_ptr<sql::TunnelConnection> TunnelManager::createTunnel(db_mgmt_Conne
       bec::GRTManager::get()->replace_status_text("Existing SSH tunnel found, connecting...");
       logInfo("Existing SSH tunnel found, connecting\n");
       config.localport = tunnel_port;
-      return std::shared_ptr<sql::TunnelConnection>(new ::SSHTunnel(this, tunnel_port, config));
+      return std::shared_ptr<SSHTunnel>(new ::SSHTunnel(this, config));
     } else {
       bool resetPassword = false;
 
@@ -196,7 +150,7 @@ std::shared_ptr<sql::TunnelConnection> TunnelManager::createTunnel(db_mgmt_Conne
             bec::GRTManager::get()->replace_status_text("SSH tunnel opened");
             logInfo("SSH tunnel opened on port: %d\n", (int )port);
             config.localport = port;
-            return std::shared_ptr<sql::TunnelConnection>(new ::SSHTunnel(this, port, config));
+            return std::shared_ptr<SSHTunnel>(new ::SSHTunnel(this, config));
           }
           case ssh::SSHReturnType::INVALID_AUTH_DATA: {
             std::string errorMsg = std::get<1>(retVal);
@@ -220,8 +174,8 @@ std::shared_ptr<sql::TunnelConnection> TunnelManager::createTunnel(db_mgmt_Conne
           case ssh::SSHReturnType::FINGERPRINT_MISMATCH: {
             std::string fingerprint = std::get<1>(retVal);
             std::string errorMsg =
-                "WARNING: Server public key has changed. It means either you're under attack or the administrator has changed the key. New public fingerprint is: "
-                    + fingerprint;
+              "WARNING: Server public key has changed. It means either you're under attack or the administrator has "
+              "changed the key. New public fingerprint is: " + fingerprint;
             mforms::Utilities::show_error("Could not connect the SSH Tunnel", errorMsg, _("Ok"));
             logDebug("Tunnel auth error, key fingerprint mismatch\n");
             throw grt::user_cancelled("");
@@ -245,9 +199,11 @@ std::shared_ptr<sql::TunnelConnection> TunnelManager::createTunnel(db_mgmt_Conne
       }
     }
   }
-  return std::shared_ptr<sql::TunnelConnection>();
+  return std::shared_ptr<SSHTunnel>();
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 TunnelManager::~TunnelManager() {
   shutdown();
   if (_manager != nullptr) {
@@ -257,3 +213,5 @@ TunnelManager::~TunnelManager() {
     delete _manager;
   }
 }
+
+//----------------------------------------------------------------------------------------------------------------------
