@@ -1,20 +1,25 @@
 /*
- * Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2024, Oracle and/or its affiliates. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 of the
- * License.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301  USA
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
  */
 
 #import "WBSidebarPanel.h"
@@ -47,6 +52,7 @@
 
   mLastSecondarySidebarWidth = MAX(bec::GRTManager::get()->get_app_option_int(mOptionName+":SecondarySidebarWidth", 220), 100);
   mLastSidebarWidth = MAX(bec::GRTManager::get()->get_app_option_int(mOptionName+":SidebarWidth", 220), MIN_SIDEBAR_WIDTH);
+  self.splitView.clipsToBounds = YES;
 
   if (mSidebarHidden)
   {
@@ -107,12 +113,14 @@
 
   {
     BOOL newCollapseState = [self.splitView isSubviewCollapsed: sidebar];
-    BOOL hidden = !mToolbar->get_item_checked("wb.toggleSidebar");
-
-    if (newCollapseState != hidden)
-    {
-      bec::GRTManager::get()->set_app_option(mOptionName+":SidebarHidden", grt::IntegerRef(newCollapseState));
-      mToolbar->set_item_checked("wb.toggleSidebar", newCollapseState);
+    if (mToolbar) {
+      BOOL hidden = !mToolbar->get_item_checked("wb.toggleSidebar");
+      
+      if (newCollapseState != hidden)
+      {
+        bec::GRTManager::get()->set_app_option(mOptionName+":SidebarHidden", grt::IntegerRef(newCollapseState));
+        mToolbar->set_item_checked("wb.toggleSidebar", newCollapseState);
+      }
     }
     if (!newCollapseState)
     {
@@ -136,12 +144,14 @@
     }
     {
       BOOL newCollapseState = [self.splitView isSubviewCollapsed: secondarySidebar];
-      BOOL hidden = !mToolbar->get_item_checked("wb.toggleSecondarySidebar");
-
-      if (newCollapseState != hidden)
-      {
-        bec::GRTManager::get()->set_app_option(mOptionName+":SecondarySidebarHidden", grt::IntegerRef(newCollapseState));
-        mToolbar->set_item_checked("wb.toggleSecondarySidebar", !newCollapseState);
+      if (mToolbar) {
+        BOOL hidden = !mToolbar->get_item_checked("wb.toggleSecondarySidebar");
+        
+        if (newCollapseState != hidden)
+        {
+          bec::GRTManager::get()->set_app_option(mOptionName+":SecondarySidebarHidden", grt::IntegerRef(newCollapseState));
+          mToolbar->set_item_checked("wb.toggleSecondarySidebar", !newCollapseState);
+        }
       }
       if (!newCollapseState)
       {
