@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2024, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms, as
  * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -18,19 +19,27 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "driver_manager.h"
 
+#include "wb_tunnel.h"
+
 #include "mysql_driver.h"
 #include "cppconn/driver.h"
 #include "cppconn/statement.h"
 #include "cppconn/exception.h"
 #include "cppconn/metadata.h"
 #include "base/string_utilities.h"
+#include "base/file_utilities.h"
+#include "grt.h"
+#include "mforms/app.h"
 
 #include <gmodule.h>
+#include <stdlib.h>
+
+using namespace wb;
 
 namespace sql {
 
@@ -135,7 +144,7 @@ namespace sql {
     return dm;
   }
 
-  DriverManager::DriverManager() : _driver_path("."), _cacheTime(0) {
+  DriverManager::DriverManager() : _driver_path("."), _cacheTime(0), _testing(false) {
   }
 
   void DriverManager::setTunnelFactoryFunction(TunnelFactoryFunction function) {
@@ -154,17 +163,17 @@ namespace sql {
     _driver_path = path;
   }
 
-  std::shared_ptr<TunnelConnection> DriverManager::getTunnel(const db_mgmt_ConnectionRef &connectionProperties) {
+  std::shared_ptr<SSHTunnel> DriverManager::getTunnel(const db_mgmt_ConnectionRef &connectionProperties) {
     db_mgmt_DriverRef drv = connectionProperties->driver();
     if (!drv.is_valid())
       throw SQLException("Invalid connection settings: undefined connection driver");
 
     if (_createTunnel)
       return _createTunnel(connectionProperties);
-    return std::shared_ptr<TunnelConnection>();
+    return std::shared_ptr<SSHTunnel>();
   }
 
-//--------------------------------------------------------------------------------------------------
+  //--------------------------------------------------------------------------------------------------
 
 #define MYSQL_PASSWORD_CACHE_TIMEOUT 60
 
@@ -174,7 +183,7 @@ namespace sql {
     if (!drv.is_valid())
       throw SQLException("Invalid connection settings: undefined connection driver");
 
-    std::shared_ptr<TunnelConnection> tunnel;
+    std::shared_ptr<SSHTunnel> tunnel;
     if (_createTunnel) {
       tunnel = _createTunnel(connectionProperties);
 
@@ -194,6 +203,17 @@ namespace sql {
       it.second();
   }
 
+  void DriverManager::set_testing() {
+    _testing = true;
+  }
+
+  //--------------------------------------------------------------------------------------------------
+
+  unsigned int DriverManager::getClientLibVersionNumeric(Driver *driver) {
+    assert(driver != NULL);
+    return driver->getMajorVersion() * 10000 + driver->getMinorVersion() * 100 + driver->getPatchVersion();
+  }
+
   //--------------------------------------------------------------------------------------------------
 
   void DriverManager::getClientLibVersion(Driver *driver) {
@@ -212,11 +232,9 @@ namespace sql {
   //--------------------------------------------------------------------------------------------------
 
   ConnectionWrapper DriverManager::getConnection(const db_mgmt_ConnectionRef &connectionProperties,
-                                                 std::shared_ptr<TunnelConnection> tunnel, Authentication::Ref password,
+                                                 std::shared_ptr<SSHTunnel> tunnel, Authentication::Ref password,
                                                  ConnectionInitSlot connection_init_slot) {
     grt::DictRef parameter_values = connectionProperties->parameterValues();
-    if (parameter_values.get_string("userName").empty())
-      throw SQLException("No user name set for this connection");
 
     // Load the driver for the connection dynamically. However for the C++ connector we have a static
     // link anyway, so use this instead.
@@ -242,6 +260,10 @@ namespace sql {
       library.append(".so");
 #endif
 
+      if (drv->name() != "MysqlNativeSaslKerberos" && drv->name() != "MysqlNativeKerberos" &&
+          parameter_values.get_string("userName").empty())
+        throw SQLException("No user name set for this connection");
+
       // 1. find driver
       GModule *gmodule = g_module_open((_driver_path + "/" + library).c_str(), G_MODULE_BIND_LOCAL);
       if (NULL == gmodule) {
@@ -301,6 +323,40 @@ namespace sql {
         properties["OPT_CONNECT_TIMEOUT"] = conn_timeout;
       if (properties.find("OPT_READ_TIMEOUT") == properties.end())
         properties["OPT_READ_TIMEOUT"] = read_timeout;
+#ifdef _MSC_VER
+      properties["pluginDir"] = base::dirname(mforms::App::get()->get_executable_path("base.dll"));
+#elif defined(__APPLE__)
+      auto nativePasswordDir = base::dirname(mforms::App::get()->get_executable_path("mysql_native_password.so"));
+      properties["pluginDir"] = nativePasswordDir + "/../Frameworks";
+#else
+      if (!_testing) {
+        properties["pluginDir"] = base::dirname(mforms::App::get()->get_executable_path("mysql_native_password.so"));
+      }
+#endif
+      properties["OPT_AUTHENTICATION_KERBEROS_CLIENT_MODE"] = "";
+      std::string krb5 = parameter_values.get_string("krb5");
+      std::string krb5cache = parameter_values.get_string("krb5cache");
+      std::vector<char> env;
+      if (!krb5.empty()) {
+        auto tmp = std::string("KRB5_CONFIG=" + krb5);
+        env = std::vector<char>(tmp.begin(), tmp.end());
+      } else {
+        auto tmp = std::string("KRB5_CONFIG=");
+        env = std::vector<char>(tmp.begin(), tmp.end());
+      }
+      putenv(&env[0]);
+
+      env.clear();
+      if (!krb5cache.empty()) {
+        auto tmp = std::string("KRB5CCNAME=" + krb5cache);
+        env = std::vector<char>(tmp.begin(), tmp.end());
+      } else {
+        auto tmp = std::string("KRB5CCNAME=");
+        env = std::vector<char>(tmp.begin(), tmp.end());
+      }
+      putenv(&env[0]);
+
+      properties["defaultAuth"] = "";
     }
     properties["OPT_CAN_HANDLE_EXPIRED_PASSWORDS"] = true;
     properties["CLIENT_MULTI_STATEMENTS"] = true;
@@ -313,7 +369,7 @@ namespace sql {
       attribs["program_name"] = "MySQLWorkbench";
       properties["OPT_CONNECT_ATTR_ADD"] = attribs;
     }
-    
+
     // If SSL is enabled but there's no certificate or anything, create the sslKey option to force enabling SSL without
     // a key
     // (equivalent to starting cmdline client with mysql --ssl-key=)
@@ -331,11 +387,10 @@ namespace sql {
 
     ssize_t sslModeWb = parameter_values.get_int("useSSL", 0);
     sql::ssl_mode sslMode = sql::SSL_MODE_DISABLED;
-    switch(sslModeWb)
-    {
+    switch (sslModeWb) {
       case 0:
-         sslMode = sql::SSL_MODE_DISABLED;
-         properties["OPT_GET_SERVER_PUBLIC_KEY"] = true;
+        sslMode = sql::SSL_MODE_DISABLED;
+        properties["OPT_GET_SERVER_PUBLIC_KEY"] = true;
         break;
       case 1:
         sslMode = sql::SSL_MODE_PREFERRED;
@@ -365,11 +420,43 @@ namespace sql {
 #else
       properties["hostName"] = std::string();
 #endif
+    } else if (drv->name() == "MysqlNativeSaslKerberos") {
+      properties["defaultAuth"] = "authentication_ldap_sasl_client";
+      std::string plugin_dir_path = parameter_values.get_string("mysqlplugindir");
+      if (!plugin_dir_path.empty()) {
+        properties["pluginDir"] = plugin_dir_path;
+      } else {
+        std::string libName = "authentication_ldap_sasl_client";
+#ifdef _MSC_VER
+        libName.append(".dll");
+#else
+        libName.append(".so");
+#endif
+        properties["pluginDir"] = base::dirname(mforms::App::get()->get_executable_path(libName));
+      }
+    } else if (drv->name() == "MysqlNativeKerberos") {
+      properties["defaultAuth"] = "authentication_kerberos_client";
+      std::string plugin_dir_path = parameter_values.get_string("mysqlplugindir");
+      if (!plugin_dir_path.empty()) {
+        properties["pluginDir"] = plugin_dir_path;
+      } else {
+        std::string libName = "authentication_kerberos_client";
+#ifdef _MSC_VER
+        libName.append(".dll");
+#else
+        libName.append(".so");
+#endif
+        properties["pluginDir"] = base::dirname(mforms::App::get()->get_executable_path(libName));
+      }
+
+      properties["OPT_AUTHENTICATION_KERBEROS_CLIENT_MODE"] = parameter_values.get_int("kerberosMode", 0) == 1 ? "SSPI": "GSSAPI";
+    } else if (drv->name() == "MysqlNativeLDAP") {
+      properties["OPT_ENABLE_CLEARTEXT_PLUGIN"] = true;
     }
 
     if (tunnel) {
       // Make the driver connect to the local tunnel port.
-      properties["port"] = tunnel->get_port();
+      properties["port"] = tunnel->getConfig().localport;
       properties["hostName"] = sql::SQLString("127.0.0.1");
     } else
       properties["hostName"] = "[" + parameter_values.get_string("hostName") +
@@ -381,7 +468,7 @@ namespace sql {
     // If we get an auth error, then we ask for the password
     bool force_ask_password = false;
   retry:
-    if (password) {
+    if (password && drv->name() != "MysqlNativeSaslKerberos" && drv->name() != "MysqlNativeKerberos") {
       authref = password;
       if (password->is_valid())
         properties["password"] = std::string(authref->password());
@@ -463,12 +550,14 @@ namespace sql {
         connection_init_slot(conn.get(), connectionProperties);
 
       //  We could set this on the parameters, but we wouldn't know the server version
-      if( conn->getMetaData()->getDatabaseMajorVersion() >= 8 ) {
-        conn.get()->createStatement()->executeUpdate("set character_set_client = utf8mb4");
-        conn.get()->createStatement()->executeUpdate("set character_set_connection = utf8mb4");
-        conn.get()->createStatement()->executeUpdate("set character_set_results = utf8mb4");
+      std::unique_ptr<sql::Statement> stmt(conn.get()->createStatement());
+      std::unique_ptr<sql::ResultSet> res(stmt->executeQuery("show character set where charset = 'utf8mb4'"));
+      if (res->rowsCount() >= 1) {
+        stmt->executeUpdate("SET NAMES 'utf8mb4'");
+      } else {
+        stmt->executeUpdate("SET NAMES 'utf8'");
       }
-      
+
       std::string def_schema = parameter_values.get_string("schema", "");
       if (!def_schema.empty())
         conn->setSchema(def_schema);
@@ -476,9 +565,13 @@ namespace sql {
       return ConnectionWrapper(std::move(conn), tunnel);
     } catch (sql::SQLException &exc) {
       // authentication error
-      if (exc.getErrorCode() == 1045 || exc.getErrorCode() == 1044 ||
-          exc.getErrorCode() == 1968 // ER_ACCESS_DENIED_NO_PASSWORD_ERROR
-          ) {
+      if (exc.getErrorCode() == 0 && getClientLibVersionNumeric(driver) >= 80019) {
+        throw sql::SQLException(exc.what(), exc.getSQLStateCStr(),
+                                2003); //  Convert to to error 2003 as the previous connector
+      } else if (exc.getErrorCode() == 1045 
+                || exc.getErrorCode() == 1044 
+                || exc.getErrorCode() == 1968 // ER_ACCESS_DENIED_NO_PASSWORD_ERROR
+                || (exc.getErrorCode() == 2000 && drv->name() == "MysqlNativeKerberos")) {
         if (!force_ask_password) {
           if (authref) {
             authref->invalidate();
