@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2018 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2021 Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -176,7 +176,7 @@ namespace mforms {
 
     // @p parent_type is only required on GTK 3.2 to 3.6, and only on the first call
     GType mforms_object_accessible_get_type(GType parent_type G_GNUC_UNUSED) {
-      static volatile gsize typeIdResult = 0;
+      static gsize typeIdResult = 0;
 
       if (g_once_init_enter(&typeIdResult)) {
         GTypeInfo tinfo = { 0, /* class size */
@@ -340,7 +340,13 @@ namespace mforms {
     gboolean mformsGTKAccessible::AtkComponentIface::grabFocus(AtkComponent *component) {
       auto *thisAccessible = FromAccessible(reinterpret_cast<GtkAccessible*>(component));
       if (thisAccessible != nullptr) {
-        return thisAccessible->_mformsAcc->accessibilityGrabFocus();
+        auto mGtk = mformsGTK::FromWidget(gtk_accessible_get_widget(GTK_ACCESSIBLE(component)));
+        GtkWidget *widget = mGtk->_windowMain;
+        bool ret = thisAccessible->_mformsAcc->accessibilityGrabFocus();
+        if (ret) {
+          gtk_widget_grab_focus(widget);
+        }
+        return ret;
       }
       return FALSE;
     }
@@ -393,7 +399,7 @@ namespace mforms {
       auto mformsGtkAcc = FromAccessible(accessible);
       if (mformsGtkAcc != nullptr && mformsGtkAcc->_mformsAcc != nullptr) {
         if (mformsGtkAcc->_name.empty())
-          mformsGtkAcc->_name = mformsGtkAcc->_mformsAcc->getAccessibilityName();
+          mformsGtkAcc->_name = mformsGtkAcc->_mformsAcc->getAccessibilityDescription();
 
         if (!mformsGtkAcc->_name.empty())
           return mformsGtkAcc->_name.c_str();
