@@ -5,12 +5,13 @@
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms, as
  * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -18,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "base/string_utilities.h"
@@ -356,7 +357,7 @@ void HomeScreen::updateColors() {
   } else {
     set_back_color("#ffffff");
   }
-  
+
 #ifdef __APPLE__
   _sidebarSection->set_back_color("#37393a");
 #else
@@ -482,8 +483,10 @@ void HomeScreen::setup_done() {
 
 void HomeScreen::showSection(size_t index) {
   if (index < _sections.size()) {
-    _sidebarSection->setActive(_sections[index]);
-    _sidebarSection->getActive()->setFocus();
+    if (_sidebarSection != nullptr) {
+      _sidebarSection->setActive(_sections[index]);
+      _sidebarSection->getActive()->setFocus();
+    }
   }
 }
 
