@@ -5,12 +5,13 @@
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms, as
  * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -35,6 +36,7 @@
 #include <stdexcept>
 #include <boost/function.hpp>
 #include <libxml/xmlmemory.h>
+#include <libxml/tree.h>
 #include "base/threading.h"
 #include <string>
 #include <gmodule.h>
@@ -2478,7 +2480,7 @@ namespace grt {
      *
      * @param requires list of other XML files required by the loaded one
      */
-    void load_metaclasses(const std::string &file, std::list<std::string> *requires = 0);
+    void load_metaclasses(const std::string &file, std::list<std::string> * requiresList = 0);
 
     /**
      * This one should not be used during normal workbench run,
@@ -2500,7 +2502,7 @@ namespace grt {
      * @param requires optional pointer to a multimap where required XML files for each loaded
      * files is stored
      */
-    int scan_metaclasses_in(const std::string &dir, std::multimap<std::string, std::string> *requires = 0);
+    int scan_metaclasses_in(const std::string &dir, std::multimap<std::string, std::string> *requiresMap = 0);
     /** End loading of metaclass definition files.
      * Finishes up loading of metaclass definition XMLs files. This will
      * check that all metaclasses referred by something were loaded. It will
