@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms, as
  * designated in a particular file or component or in included license
  * documentation. The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
@@ -35,7 +36,8 @@ using namespace parsers;
 using namespace antlr4;
 using namespace antlr4::atn;
 
-static std::string last_error;
+static std::string lastErrors;
+
 NSString *sql1 = @"select 2 as expected, /*!01000/**/*/ 2 as result";
 NSString *sql2 = @"select (select\n t1.id as a, sakila.actor.actor_id b, t2.id c, "
                   "(select  1 * 0.123, a from t3) from  `ÄÖÜ丈` t1, sakila.actor as t2\n"
@@ -159,7 +161,9 @@ public:
   virtual void syntaxError(Recognizer *recognizer, antlr4::Token *offendingSymbol, size_t line,
                            size_t charPositionInLine, const std::string &msg, std::exception_ptr e) override {
     // Here we use the message provided by the DefaultErrorStrategy class.
-    last_error = "line " + std::to_string(line) + ":" + std::to_string(charPositionInLine) + " " + msg;
+    if (!lastErrors.empty())
+      lastErrors += "\n";
+    lastErrors += "line " + std::to_string(line) + ":" + std::to_string(charPositionInLine) + " " + msg;
   }
 };
 
@@ -288,29 +292,29 @@ static Ref<BailErrorStrategy> errorStrategy = std::make_shared<BailErrorStrategy
   [defaults setObject:singleQueryText.string forKey: @"single-query"];
   [defaults setObject:versionText.stringValue forKey: @"version"];
 
-  last_error = "";
+  lastErrors = "";
   errorText.string = @"";
-  [errorText setNeedsDisplay:YES];
+  [errorText setNeedsDisplay: YES];
   output.string = @"";
-  [output setNeedsDisplay:YES];
+  [output setNeedsDisplay: YES];
   parseTreeView.string = @"";
-  [parseTreeView setNeedsDisplay:YES];
+  [parseTreeView setNeedsDisplay: YES];
 
-  size_t errorCount = [self parseQuery:singleQueryText.string
+  size_t errorCount = [self parseQuery: singleQueryText.string
                                version: [self getServerVersion]
                                  modes: [self getSqlModes]
-                          dumpToOutput:YES
-                              needTree:YES];
+                          dumpToOutput: YES
+                              needTree: YES];
 
   NSString *combinedErrorText = [NSString stringWithFormat: @"Parse time: %.6fs\n\n", lastDuration];
 
   if (errorCount > 0) {
     combinedErrorText =
-      [combinedErrorText stringByAppendingFormat: @"%zu errors found\n%s", errorCount, last_error.c_str()];
+      [combinedErrorText stringByAppendingFormat: @"%zu errors found\n%s", errorCount, lastErrors.c_str()];
   } else {
     combinedErrorText = [combinedErrorText stringByAppendingString: @"No errors found"];
   }
-  [errorText setString:combinedErrorText];
+  [errorText setString: combinedErrorText];
 }
 
 - (IBAction)selectFile: (id)sender {
@@ -352,7 +356,7 @@ static Ref<BailErrorStrategy> errorStrategy = std::make_shared<BailErrorStrategy
 
   stopTests = NO;
   errorQueryText.string = @"";
-  last_error = "";
+  lastErrors = "";
 
   statusText.stringValue = @"Counting queries...";
   stepsList.string = @"Counting queries\n";
