@@ -1,10 +1,13 @@
 variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('group_replication_start_on_boot', ['Replication/Group']),
   ('group_replication_local_address', ['Replication/Group']),
+  ('group_replication_member_weight', ['Replication/Group']),
   ('group_replication_group_seeds', ['Replication/Group']),
   ('group_replication_force_members', ['Replication/Group']),
   ('group_replication_bootstrap_group', ['Replication/Group']),
   ('group_replication_poll_spin_loops', ['Replication/Group']),
+  ('group_replication_recovery_get_public_key', ['Replication/Group']),
+  ('group_replication_recovery_public_key_path', ['Replication/Group']),
   ('group_replication_recovery_retry_count', ['Replication/Group']),
   ('group_replication_recovery_reconnect_interval', ['Replication/Group']),
   ('group_replication_recovery_use_ssl', ['Replication/Group']),
@@ -12,10 +15,12 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('group_replication_recovery_ssl_capath', ['Replication/Group']),
   ('group_replication_recovery_ssl_cert', ['Replication/Group']),
   ('group_replication_recovery_ssl_key', ['Replication/Group']),
+  ('group_replication_recovery_ssl_cipher', ['Replication/Group']),
   ('group_replication_recovery_ssl_crl', ['Replication/Group']),
   ('group_replication_recovery_ssl_crlpath', ['Replication/Group']),
   ('group_replication_recovery_ssl_verify_server_cert', ['Replication/Group']),
   ('group_replication_recovery_complete_at', ['Replication/Group']),
+  ('group_replication_communication_debug_options', ['Replication/Group']),
   ('group_replication_components_stop_timeout', ['Replication/Group']),
   ('group_replication_allow_local_lower_version_join', ['Replication/Group']),
   ('group_replication_allow_local_disjoint_gtids_join', ['Replication/Group']),
@@ -24,38 +29,60 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('group_replication_gtid_assignment_block_size', ['Replication/Group']),
   ('group_replication_ssl_mode', ['Replication/Group']),
   ('group_replication_single_primary_mode', ['Replication/Group']),
+  ('group_replication_transaction_size_limit', ['Replication/Group']),
+  ('group_replication_unreachable_majority_timeout', ['Replication/Group']),
   ( 'group_replication_enforce_update_everywhere_checks',
     ['Replication/Group']),
   ('group_replication_flow_control_mode', ['Replication/Group']),
   ( 'group_replication_flow_control_certifier_threshold',
     ['Replication/Group']),
   ('group_replication_flow_control_applier_threshold', ['Replication/Group']),
+  ('group_replication_flow_control_hold_percent', ['Replication/Group']),
+  ('group_replication_flow_control_max_commit_quota', ['Replication/Group']),
+  ( 'group_replication_flow_control_member_quota_percent',
+    ['Replication/Group']),
+  ('group_replication_flow_control_min_quota', ['Replication/Group']),
+  ('group_replication_flow_control_min_recovery_quota', ['Replication/Group']),
+  ('group_replication_flow_control_period', ['Replication/Group']),
+  ('group_replication_flow_control_release_percent', ['Replication/Group']),
   ('group_replication_ip_whitelist', ['Replication/Group']),
-  ('mysqlx_port', ['Other/X']),
+  ('mysqlx-ssl', ['Other/X']),
+  ('mysqlx-ssl-key', ['Other/X']),
+  ('mysqlx-ssl-ca', ['Other/X']),
+  ('mysqlx-ssl-capath', ['Other/X']),
+  ('mysqlx-ssl-cert', ['Other/X']),
+  ('mysqlx-ssl-cipher', ['Other/X']),
+  ('mysqlx-ssl-crl', ['Other/X']),
+  ('mysqlx-ssl-crlpath', ['Other/X']),
+  ('mysqlx_bind_address', ['Other/X']),
+  ('mysqlx_connect_timeout', ['Other/X']),
+  ('mysqlx_document_id_unique_prefix', ['Other/X']),
+  ('mysqlx_idle_worker_thread_timeout', ['Other/X']),
+  ('mysqlx_interactive_timeout', ['Other/X']),
+  ('mysqlx_max_allowed_packet', ['Other/X']),
   ('mysqlx_max_connections', ['Other/X']),
   ('mysqlx_min_worker_threads', ['Other/X']),
-  ('mysqlx_idle_worker_thread_timeout', ['Other/X']),
-  ('mysqlx_max_allowed_packet', ['Other/X']),
-  ('mysqlx_connect_timeout', ['Other/X']),
-  ('mysqlx_ssl', ['Other/X']),
-  ('mysqlx_ssl_key', ['Other/X']),
-  ('mysqlx_ssl_ca', ['Other/X']),
-  ('mysqlx_ssl_capath', ['Other/X']),
-  ('mysqlx_ssl_cert', ['Other/X']),
-  ('mysqlx_ssl_cipher', ['Other/X']),
-  ('mysqlx_ssl_crl', ['Other/X']),
-  ('mysqlx_ssl_crlpath', ['Other/X']),
+  ('mysqlx_port', ['Other/X']),
+  ('mysqlx_port_open_timeout', ['Other/X']),
+  ('mysqlx_read_timeout', ['Other/X']),
+  ('mysqlx_socket', ['Other/X']),
+  ('mysqlx_wait_timeout', ['Other/X']),
+  ('mysqlx_write_timeout', ['Other/X']),
   ('abort-slave-event-count', ['Replication/Slave']),
+  ('activate_all_roles_on_login', ['Security/Authentication']),
   ('allow-suspicious-udfs', ['Security/Security']),
   ('ansi', ['General/SQL']),
   ('audit-log', ['Logging/Audit']),
   ('audit_log_buffer_size', ['Logging/Audit']),
+  ('audit_log_compression', ['Logging/Audit']),
   ('audit_log_connection_policy', ['Logging/Audit']),
+  ('audit_log_encryption', ['Logging/Audit']),
   ('audit_log_exclude_accounts', ['Logging/Audit']),
   ('audit_log_file', ['Logging/Audit']),
   ('audit_log_format', ['Logging/Audit']),
   ('audit_log_include_accounts', ['Logging/Audit']),
   ('audit_log_policy', ['Logging/Audit']),
+  ('audit_log_read_buffer_size', ['Logging/Audit']),
   ('audit_log_rotate_on_size', ['Logging/Audit']),
   ('audit_log_statement_policy', ['Logging/Audit']),
   ('audit_log_strategy', ['Logging/Audit']),
@@ -63,7 +90,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('authentication_windows_use_principal_name', ['Security/Security']),
   ('auto_generate_certs', ['Security/Security']),
   ('autocommit', ['General/Transactions']),
-  ('avoid_temporal_upgrade', []),
+  ('avoid_temporal_upgrade', ['General/System']),
   ('backup_elevation', ['General/Backup']),
   ('backup_history_log', ['General/Backup']),
   ('backup_history_log_file', ['General/Backup']),
@@ -78,34 +105,44 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('binlog-format', ['Logging/Binlog Options']),
   ('binlog-ignore-db', ['Logging/Binlog Options']),
   ('binlog-row-event-max-size', ['Logging/Binlog Options']),
-  ('binlog-rows-query-log-events', ['Logging/Binlog Options']),
   ('binlog_cache_size', ['Logging/Binlog Options']),
   ('binlog_direct_non_transactional_updates', ['Logging/Binlog Options']),
   ('binlog_error_action', ['Logging/Binlog Options']),
+  ('binlog_expire_logs_seconds', ['Logging/Binlog Options']),
   ('binlog_group_commit_sync_delay', ['Logging/Binlog Options']),
   ('binlog_group_commit_sync_no_delay_count', ['Logging/Binlog Options']),
   ('binlog_gtid_simple_recovery', ['Logging/Binlog Options']),
   ('binlog_row_image', ['Logging/Binlog Options']),
+  ('binlog_row_metadata', ['Logging/Binlog Options']),
+  ('binlog_row_value_options', ['Logging/Binlog Options']),
+  ('binlog_rows_query_log_events', ['Logging/Binlog Options']),
   ('binlog_stmt_cache_size', ['Logging/Binlog Options']),
+  ('binlog_transaction_dependency_tracking', ['Logging/Binlog Options']),
+  ('binlog_transaction_dependency_history_size', ['Logging/Binlog Options']),
   ('binlogging_impossible_mode', ['Logging/Binlog Options']),
   ('block_encryption_mode', ['Security/Security']),
   ('bootstrap', ['Advanced/General']),
   ('bulk_insert_buffer_size', ['Advanced/Various']),
+  ('caching_sha2_password_auto_generate_rsa_keys', ['Security/Security']),
+  ('caching_sha2_password_private_key_path', ['Security/Security']),
+  ('caching_sha2_password_public_key_path', ['Security/Security']),
   ('character-set-filesystem', ['General/International']),
   ('character-set-server', ['General/International']),
   ('character-sets-dir', ['General/International']),
-  ('check_proxy_users', []),
+  ('check_proxy_users', ['Security/Authentication']),
   ('chroot', ['Advanced/General']),
   ('collation-server', ['General/International']),
   ('completion_type', ['Advanced/Transactions']),
   ('concurrent_insert', ['MyISAM/General']),
-  ('connection_control_failed_connections_threshold', []),
-  ('connection_control_max_connection_delay', []),
-  ('connection_control_min_connection_delay', []),
+  ( 'connection_control_failed_connections_threshold',
+    ['Security/Authentication']),
+  ('connection_control_max_connection_delay', ['Security/Authentication']),
+  ('connection_control_min_connection_delay', ['Security/Authentication']),
   ('connect_timeout', ['Networking/Timeout Settings']),
   ('console', ['Logging/General']),
   ('core-file', ['General/System']),
-  ('create_old_temporals', []),
+  ('create_old_temporals', ['Other/NDB']),
+  ('cte_max_recursion_depth', ['General/SQL']),
   ('daemon_memcached_enable_binlog', ['Other/Memcached']),
   ('daemon_memcached_engine_lib_name', ['Other/Memcached']),
   ('daemon_memcached_engine_lib_path', ['Other/Memcached']),
@@ -118,6 +155,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('debug-sync-timeout', ['Advanced/General']),
   ('default-character-set', ['General/International']),
   ('default-collation', ['General/International']),
+  ('default_collation_for_utf8mb4', ['General/International']),
   ('default-storage-engine', ['General/General']),
   ('default-table-type', ['General/General']),
   ('default-time-zone', ['General/International']),
@@ -131,11 +169,12 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('delayed_queue_size', ['Advanced/Insert delayed settings']),
   ('des-key-file', ['Security/Security']),
   ('disable_gtid_unsafe_statements', ['Replication/General']),
-  ('disable-partition-engine-check', []),
-  ('disabled_storage_engines', []),
+  ('disable-partition-engine-check', ['General/System']),
+  ('disabled_storage_engines', ['General/System']),
   ('disconnect-slave-event-count', ['Replication/Slave']),
   ('disconnect_on_expired_password', ['Security/Authentication']),
   ('div_precision_increment', ['Advanced/General']),
+  ('dragnet.log_error_filter_rules', ['Logging/Advanced log options']),
   ('early-plugin-load', ['Advanced/General']),
   ('enable-locking', ['General/Features']),
   ('enable-named-pipe', ['Networking/General']),
@@ -160,11 +199,12 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('general-log', ['Logging/General']),
   ('general_log_file', ['Logging/General']),
   ('group_concat_max_len', ['Advanced/Various']),
-  ('gtid-executed-compression-period', []),
+  ('gtid-executed-compression-period', ['Advanced/General']),
   ('gtid-mode', ['Replication/General']),
+  ('histogram_generation_max_mem_size', ['Performance/Optimizer']),
   ('ignore-builtin-innodb', ['InnoDB/General']),
   ('ignore-db-dir', ['General/System']),
-  ('information_schema_stats', []),
+  ('information_schema_stats_expiry', ['Performance/Optimizer']),
   ('init-file', ['General/General']),
   ('init-rpl-role', ['Replication/General']),
   ('init_connect', ['Advanced/General']),
@@ -188,7 +228,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('innodb_background_drop_list_empty', ['InnoDB/General']),
   ('innodb_buffer_pool_awe_mem_mb', ['InnoDB/Buffer pool']),
   ('innodb_buffer_pool_chunk_size', ['InnoDB/Buffer pool']),
-  ('innodb_buffer_pool_debug', []),
+  ('innodb_buffer_pool_debug', ['InnoDB/Buffer pool']),
   ('innodb_buffer_pool_dump_at_shutdown', ['InnoDB/Buffer pool']),
   ('innodb_buffer_pool_dump_now', ['InnoDB/Buffer pool']),
   ('innodb_buffer_pool_dump_pct', ['InnoDB/General']),
@@ -201,6 +241,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('innodb_change_buffer_max_size', ['InnoDB/General']),
   ('innodb_change_buffering', ['InnoDB/General']),
   ('innodb_change_buffering_debug', ['InnoDB/General']),
+  ('innodb_checkpoint_disabled', ['InnoDB/General']),
   ('innodb_checksum_algorithm', ['InnoDB/General']),
   ('innodb_checksums', ['InnoDB/General']),
   ('innodb_cmp_per_index_enabled', ['InnoDB/General']),
@@ -210,17 +251,19 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('innodb_compression_level', ['InnoDB/General']),
   ('innodb_compression_pad_pct_max', ['InnoDB/General']),
   ('innodb_concurrency_tickets', ['InnoDB/General']),
-  ('innodb_create_intrinsic', ['InnoDB/General']),
   ('innodb_data_file_path', ['InnoDB/Datafiles']),
   ('innodb_data_home_dir', ['InnoDB/Datafiles']),
-  ('innodb_deadlock_detect', []),
+  ('innodb_ddl_log_crash_reset_debug', ['InnoDB/Logfiles']),
+  ('innodb_deadlock_detect', ['InnoDB/General']),
+  ('innodb_dedicated_server', ['InnoDB/General']),
   ('innodb_default_row_format', ['InnoDB/General']),
+  ('innodb_directories', ['InnoDB/General']),
   ('innodb_disable_sort_file_cache', ['InnoDB/General']),
   ('innodb_disable_resize_buffer_pool_debug', ['InnoDB/General']),
   ('innodb_doublewrite', ['InnoDB/General']),
   ('innodb_extra_dirty_writes', ['InnoDB/General']),
   ('innodb_fast_shutdown', ['InnoDB/Various']),
-  ('innodb_fil_make_page_dirty_debug', []),
+  ('innodb_fil_make_page_dirty_debug', ['InnoDB/General']),
   ('innodb_file_format', ['InnoDB/Datafiles']),
   ('innodb_file_format_check', ['InnoDB/Datafiles']),
   ('innodb_file_format_max', ['InnoDB/Datafiles']),
@@ -261,6 +304,9 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('innodb_log_file_size', ['InnoDB/Logfiles']),
   ('innodb_log_files_in_group', ['InnoDB/Logfiles']),
   ('innodb_log_group_home_dir', ['InnoDB/Logfiles']),
+  ('innodb_log_spin_cpu_abs_lwm', ['InnoDB/Logfiles']),
+  ('innodb_log_spin_cpu_pct_hwm', ['InnoDB/Logfiles']),
+  ('innodb_log_wait_for_flush_spin_hwm', ['InnoDB/Logfiles']),
   ('innodb_log_write_ahead_size', ['InnoDB/Logfiles']),
   ('innodb_lru_scan_depth', ['InnoDB/General']),
   ('innodb_max_dirty_pages_pct', ['InnoDB/General']),
@@ -287,6 +333,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('innodb_page_cleaners', ['InnoDB/Memory']),
   ('innodb_page_size', ['InnoDB/Memory']),
   ('innodb_print_all_deadlocks', ['InnoDB/General']),
+  ('innodb_print_ddl_logs', ['InnoDB/General']),
   ('innodb_purge_batch_size', ['InnoDB/General']),
   ('innodb_purge_rseg_truncate_frequency', ['InnoDB/General']),
   ('innodb_purge_threads', ['InnoDB/General']),
@@ -294,10 +341,12 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('innodb_read_ahead_threshold', ['InnoDB/General']),
   ('innodb_read_io_threads', ['InnoDB/General']),
   ('innodb_read_only', ['InnoDB/General']),
+  ('innodb_redo_log_encrypt', ['InnoDB/General']),
   ('innodb_replication_delay', ['InnoDB/General']),
   ('innodb_rollback_on_timeout', ['InnoDB/General']),
   ('innodb_rollback_segments', ['InnoDB/General']),
   ('innodb_saved_page_number_debug', ['InnoDB/General']),
+  ('innodb_scan_directories', ['InnoDB/General']),
   ('innodb_sort_buffer_size', ['InnoDB/General', 'InnoDB/Memory']),
   ('innodb_spin_wait_delay', ['InnoDB/General']),
   ('innodb_stats_auto_recalc', ['InnoDB/General']),
@@ -317,6 +366,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('innodb_sync_debug', ['InnoDB/General']),
   ('innodb_table_locks', ['InnoDB/General']),
   ('innodb_temp_data_file_path', ['InnoDB/General']),
+  ('internal_tmp_mem_storage_engine', ['InnoDB/General']),
   ('innodb_tmpdir', ['InnoDB/General']),
   ('innodb_thread_concurrency', ['InnoDB/General']),
   ('innodb_thread_concurrency_timer_based', ['InnoDB/General']),
@@ -324,6 +374,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('innodb_trx_purge_view_update_only_debug', ['InnoDB/General']),
   ('innodb_trx_rseg_n_slots_debug', ['InnoDB/General']),
   ('innodb_undo_directory', ['InnoDB/General', 'InnoDB/Logfiles']),
+  ('innodb_undo_log_encrypt', ['InnoDB/General', 'InnoDB/Logfiles']),
   ('innodb_undo_log_truncate', ['InnoDB/General', 'InnoDB/Logfiles']),
   ('innodb_undo_logs', ['InnoDB/General', 'InnoDB/Logfiles']),
   ('innodb_undo_tablespaces', ['InnoDB/General']),
@@ -341,12 +392,56 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('key_cache_age_threshold', ['MyISAM/General']),
   ('key_cache_block_size', ['Advanced/Various']),
   ('key_cache_division_limit', ['Advanced/Various']),
-  ('keyring_file_data', []),
-  ('keyring_okv_conf_dir', []),
+  ('keyring_aws_cmk_id', ['Security/Authentication']),
+  ('keyring_aws_conf_file', ['Security/Authentication']),
+  ('keyring_aws_data_file', ['Security/Authentication']),
+  ('keyring_aws_region', ['Security/Authentication']),
+  ('keyring_encrypted_file_data', ['Security/Authentication']),
+  ('keyring_encrypted_file_password', ['Security/Authentication']),
+  ('keyring_file_data', ['Security/Authentication']),
+  ('keyring-migration-destination', ['Security/Authentication']),
+  ('keyring-migration-host', ['Security/Authentication']),
+  ('keyring-migration-password', ['Security/Authentication']),
+  ('keyring-migration-port', ['Security/Authentication']),
+  ('keyring-migration-socket', ['Security/Authentication']),
+  ('keyring-migration-source', ['Security/Authentication']),
+  ('keyring-migration-user', ['Security/Authentication']),
+  ('keyring_okv_conf_dir', ['Security/Authentication']),
   ('language', ['General/International']),
   ('large-pages', ['General/Memory usage']),
   ('lc-messages', ['General/International']),
   ('lc-messages-dir', ['General/International']),
+  ('authentication_ldap_sasl_auth_method_name', ['Security/Authentication']),
+  ('authentication_ldap_sasl_bind_base_dn', ['Security/Authentication']),
+  ('authentication_ldap_sasl_bind_root_dn', ['Security/Authentication']),
+  ('authentication_ldap_sasl_bind_root_pwd', ['Security/Authentication']),
+  ('authentication_ldap_sasl_ca_path', ['Security/Authentication']),
+  ('authentication_ldap_sasl_group_search_attr', ['Security/Authentication']),
+  ( 'authentication_ldap_sasl_group_search_filter',
+    ['Security/Authentication']),
+  ('authentication_ldap_sasl_init_pool_size', ['Security/Authentication']),
+  ('authentication_ldap_sasl_log_status', ['Security/Authentication']),
+  ('authentication_ldap_sasl_max_pool_size', ['Security/Authentication']),
+  ('authentication_ldap_sasl_server_host', ['Security/Authentication']),
+  ('authentication_ldap_sasl_server_port', ['Security/Authentication']),
+  ('authentication_ldap_sasl_tls', ['Security/Authentication']),
+  ('authentication_ldap_sasl_user_search_attr', ['Security/Authentication']),
+  ('authentication_ldap_simple_auth_method_name', ['Security/Authentication']),
+  ('authentication_ldap_simple_bind_base_dn', ['Security/Authentication']),
+  ('authentication_ldap_simple_bind_root_dn', ['Security/Authentication']),
+  ('authentication_ldap_simple_bind_root_pwd', ['Security/Authentication']),
+  ('authentication_ldap_simple_ca_path', ['Security/Authentication']),
+  ( 'authentication_ldap_simple_group_search_attr',
+    ['Security/Authentication']),
+  ( 'authentication_ldap_simple_group_search_filter',
+    ['Security/Authentication']),
+  ('authentication_ldap_simple_init_pool_size', ['Security/Authentication']),
+  ('authentication_ldap_simple_log_status', ['Security/Authentication']),
+  ('authentication_ldap_simple_max_pool_size', ['Security/Authentication']),
+  ('authentication_ldap_simple_server_host', ['Security/Authentication']),
+  ('authentication_ldap_simple_server_port', ['Security/Authentication']),
+  ('authentication_ldap_simple_tls', ['Security/Authentication']),
+  ('authentication_ldap_simple_user_search_attr', ['Security/Authentication']),
   ('lock_wait_timeout', ['General/General']),
   ('log', ['Logging/General']),
   ('log-backup-output', ['General/Backup']),
@@ -370,6 +465,8 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('log_backward_compatible_user_definitions', ['Logging/General']),
   ('log_bin_use_v1_row_events', ['Logging/Binlog Options']),
   ('log_builtin_as_identified_by_password', ['Logging/General']),
+  ('log_error_filter_rules', ['Logging/General']),
+  ('log_error_services', ['Logging/General']),
   ('log_error_verbosity', ['Logging/General']),
   ('log_slave_updates', ['Logging/General']),
   ('log_syslog', ['Logging/General']),
@@ -380,6 +477,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('long_query_time', ['Logging/Slow query log options']),
   ('low-priority-updates', ['Performance/General']),
   ('lower_case_table_names', ['General/System']),
+  ('mandatory_roles', ['General/System']),
   ('master-connect-retry', ['Replication/Master']),
   ('master-host', ['Replication/Master']),
   ('master-info-file', ['Replication/Master']),
@@ -474,6 +572,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('ndb_log_empty_update', ['Other/NDB']),
   ('ndb_log_exclusive_reads', ['Other/NDB']),
   ('ndb_log_orig', ['Other/NDB']),
+  ('ndb-log-update-minimal', ['Other/NDB']),
   ('ndb_log_updated_only', ['Other/NDB']),
   ('ndb_optimized_node_selection', ['Other/NDB']),
   ('ndb_read_backup', ['Other/NDB']),
@@ -489,6 +588,8 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('net_write_timeout', ['Networking/Timeout Settings']),
   ('new', ['Advanced/Various']),
   ('ngram_token_size', ['InnoDB/Fulltext search']),
+  ('no-dd-upgrade', ['General/General']),
+  ('no-monitor', ['General/System']),
   ('offline_mode', ['General/System']),
   ('old', ['General/General']),
   ('old-alter-table', ['Advanced/Deprecated']),
@@ -501,7 +602,9 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('optimizer_search_depth', ['Performance/Optimizer']),
   ('optimizer_switch', ['Performance/Optimizer']),
   ('parser_max_mem_size', ['General/SQL']),
-  ('partition', []),
+  ('partition', ['General/System']),
+  ('password_history', ['Security/Authentication']),
+  ('password_reuse_interval', ['Security/Authentication']),
   ( 'performance-schema-consumer-events-stages-current',
     ['Performance/Performance Schema']),
   ( 'performance-schema-consumer-events-stages-history',
@@ -558,6 +661,8 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ( 'performance_schema_max_cond_instances',
     ['Performance/Performance Schema']),
   ('performance_schema_max_digest_length', ['Performance/Performance Schema']),
+  ( 'performance_schema_max_digest_sample_age',
+    ['Performance/Performance Schema']),
   ('performance_schema_max_file_classes', ['Performance/Performance Schema']),
   ('performance_schema_max_file_handles', ['Performance/Performance Schema']),
   ( 'performance_schema_max_file_instances',
@@ -626,6 +731,8 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('read_buffer_size', ['Advanced/Various']),
   ('read_only', ['Security/Security']),
   ('read_rnd_buffer_size', ['Advanced/Various']),
+  ('regexp_stack_limit', ['Advanced/Various']),
+  ('regexp_time_limit', ['Advanced/Various']),
   ('relay-log', ['Replication/Relay Log']),
   ('relay-log-info-repository', ['Replication/Relay Log']),
   ('relay_log_index', ['Replication/Relay Log']),
@@ -649,6 +756,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('restore_disables_events', ['Advanced/General']),
   ('restore_elevation', ['Advanced/General']),
   ('restore_precheck', ['Advanced/General']),
+  ('rpl_read_size', ['Replication/General']),
   ('rpl_stop_slave_timeout', ['Replication/Slave']),
   ('safe-mode', ['Security/Security']),
   ('safe-show-database', ['Security/Security']),
@@ -659,20 +767,23 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('secure-backup-file-priv', ['Security/Security']),
   ('secure-file-priv', ['Security/Security']),
   ('server-id', ['General/Informational']),
-  ('server_id_bits', []),
+  ('server_id_bits', ['General/System']),
   ('session_track_gtids', ['General/General']),
   ('session_track_schema', ['General/General']),
   ('session_track_state_change', ['General/General']),
   ('session_track_system_variables', ['General/General']),
   ('set-variable', ['Advanced/Deprecated']),
-  ('sha256_password_auto_generate_rsa_keys', []),
-  ('sha256_password_proxy_users', []),
-  ('shared_memory', []),
-  ('shared_memory_base_name', []),
+  ('sha256_password_auto_generate_rsa_keys', ['Security/Authentication']),
+  ('sha256_password_private_key_path', ['Security/Authentication']),
+  ('sha256_password_proxy_users', ['Security/Authentication']),
+  ('sha256_password_public_key_path', ['Security/Authentication']),
+  ('shared_memory', ['General/System']),
+  ('shared_memory_base_name', ['General/System']),
   ('show-slave-auth-info', ['Replication/Slave']),
-  ('show_compatibility_56', []),
-  ('show_old_temporals', []),
-  ('simplified_binlog_gtid_recovery', []),
+  ('show_compatibility_56', ['General/General']),
+  ('show_create_table_verbosity', ['General/General']),
+  ('show_old_temporals', ['General/General']),
+  ('simplified_binlog_gtid_recovery', ['Logging/Binlog Options']),
   ('skip-bdb', ['Other/Storage Engines']),
   ('skip-character-set-client-handshake', ['General/International']),
   ('skip-concurrent-insert', ['Advanced/General']),
@@ -723,12 +834,13 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('ssl-cipher', ['Networking/SSL']),
   ('ssl-crl', ['Networking/SSL']),
   ('ssl-crlpath', ['Networking/SSL']),
+  ('ssl_fips_mode', ['Networking/SSL']),
   ('ssl-key', ['Networking/SSL']),
   ('standalone', ['Advanced/General']),
   ('stored_program_cache', ['General/General']),
-  ('stored_program_definition_cache', []),
+  ('stored_program_definition_cache', ['General/General']),
   ('super-large-pages', ['General/General']),
-  ('super_read_only', []),
+  ('super_read_only', ['General/System']),
   ('symbolic-links', ['General/System']),
   ('sync_binlog', ['Logging/Binlog Options']),
   ('sync_frm', ['Advanced/General']),
@@ -736,11 +848,12 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('sync_relay_log', ['Replication/Relay Log']),
   ('sync_relay_log_info', ['Replication/Relay Log']),
   ('sysdate-is-now', ['General/System']),
-  ('tablespace_definition_cache', []),
-  ('table_cache', []),
-  ('table_lock_wait_timeout', []),
-  ('tc-heuristic-recover', []),
-  ('temp-pool', []),
+  ('tablespace_definition_cache', ['General/System']),
+  ('table_cache', ['General/System']),
+  ('table_lock_wait_timeout', ['General/System']),
+  ('tc-heuristic-recover', ['General/System']),
+  ('temp-pool', ['General/System']),
+  ('temptable_max_ram', ['General/System']),
   ('thread_cache_size', ['Advanced/Thread specific settings']),
   ('thread_concurrency', ['Advanced/Thread specific settings']),
   ('thread_handling', ['Advanced/Thread specific settings']),
@@ -753,7 +866,7 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('thread_pool_stall_limit', ['Advanced/Thread specific settings']),
   ('thread_stack', ['Advanced/Thread specific settings']),
   ('timed_mutexes', ['InnoDB/General']),
-  ('tls_version', []),
+  ('tls_version', ['Security/Authentication']),
   ('tmp_table_size', ['Advanced/Various']),
   ('tmpdir', ['General/Directories']),
   ('transaction-isolation', ['Advanced/Transactions']),
@@ -763,11 +876,13 @@ variable_groups = [ ('group_replication_group_name', ['Replication/Group']),
   ('updatable_views_with_limit', ['Advanced/General']),
   ('use-symbolic-links', ['General/System']),
   ('user', ['General/System']),
-  ('validate-password', []),
-  ('validate_password_check_user_name', []),
-  ('verbose', []),
-  ('version_tokens_session', []),
-  ('version_tokens_session_number', []),
+  ('validate-password', ['Security/Authentication']),
+  ('validate_password.check_user_name', ['Security/Authentication']),
+  ('validate_password_check_user_name', ['Security/Authentication']),
+  ('verbose', ['General/General']),
+  ('version_tokens_session', ['General/System']),
+  ('version_tokens_session_number', ['General/System']),
   ('wait_timeout', ['Networking/Timeout Settings']),
   ('warnings', ['Advanced/Deprecated']),
+  ('windowing_use_high_precision', ['General/System']),
   ('skip-innodb', ['InnoDB/General'])]
